#ifndef _ttail_init_h__
#define _ttail_init_h__

#include <ctype.h>
#include <errno.h>
#include <getopt.h>
#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "ttail.h"

/**@brief Parse cli arguments and return a ttail_t
 *@param int argc
 *@param char** argv
 *@return NULL on error
 */
ttail_t *ttail_init(int argc, char **argv);

/**@brief Checks that init lead to a valid runtime
 *@param ttail_t* t
 *@return 0 if no error -1 if fatal error
 */
int ttail_init_check(ttail_t*);

/**@brief Add a logfile
 *@param ttail_t*
 *@param const char * filename
 *@return 0 if no errors 1 if unable to open file -1 if fatal error
 */
int ttail_add_logfile(ttail_t*, const char*);

/**@brief Set a date prefix regex
 *@param ttail_t*
 *@param const char * regex
 *@return 0 if no errors 1 if allready set -1 if compilation fails
 */
int ttail_set_prefix(ttail_t*, const char*);

/**@brief Set dates min/max
 *
 *After the call dates are free and set to NULL except if error
 *@param ttail_t ttail instance
 *@param char*[2] dates {min,max} both can be NULL
 *@return -1 if error 0 else
 */
int ttail_set_dates(ttail_t*, char*[2]);

/**@brief Attempt to guess a dateformat
 *@param ttail_t* ttail if manage to guess set the ttail_t.fmt
 *@param const char* date as a dtring
 *@param struct tm* if non NULL will be set to detected date
 *@return -1 if no guess -2 if fmt already set else id in TTAIL_DEFAULT_FORMATS
 */
int ttail_format_guess(ttail_t*, const char*, struct tm*);


void ttail_free(ttail_t*);


#endif
