#include "ttail_search.h"

int ttail_search_closest(ttail_t* ttail)
{
	if(ttail->session != NULL)
	{
		fprintf(stderr, "A session is allready started\n");
		return -1;
	}
	/*
	ttail->session = (ttail_search_t*)malloc(ttail->logfile_sz?
		sizeof(ttail_search_file_t):sizeof(ttail_search_stdin_t));
	if(!ttail->session)
	{
		perror("Unable to allocate memory for search session");
		return -1;
	}
	*/
	return ttail->logfile_sz?\
		_ttail_search_closest_files(ttail):\
		_ttail_search_closest_stdin(ttail);
}

int _ttail_search_closest_files(ttail_t* t)
{
	return _ttail_search_closest_files_init(t);
}

int _ttail_search_closest_files_init(ttail_t* t)
{
	struct stat stat;
	FILE *fp;
	size_t i;
	off_t *file_sz;

	t->session = (ttail_search_t*)malloc(sizeof(ttail_search_file_t));
	if(!t->session)
	{
		perror("Unable to allocate memory for search session");
		goto _ttail_search_closest_files_alloc_session_err;
	}
	memset(t->session, 0, sizeof(ttail_search_file_t));

	file_sz = malloc(sizeof(off_t)*t->logfile_sz);
	if(!file_sz)
	{
		perror("Unable to allocate memory for file sizes");
		goto _ttail_search_closest_files_alloc_err;
	}
	t->session->file.file_sz = file_sz;
	#ifdef HUGEFILE
	t->session->file.sz_div = 0;
	#endif

	for(i=0;i<t->logfile_sz;i++)
	{
		fp = t->logfile[i];
		if(!fp)
		{
			if(_ttail_file_reopen(t,i))
			{
				file_sz[i] = 0;
				continue;
			}
		}
		if(fstat(fileno(fp), &stat))
		{
			perror("Unable to get file size");
			goto _ttail_search_closest_files_err;
		}
		file_sz[i] = stat.st_size;
	}
	/* we got all real size, determining if we need a divisor */
	/*
	 * not implemented
	 */
	if(_ttail_search_closest_files_set_fsizes(t))
	{
		goto _ttail_search_closest_files_err;
	}

	return 0;

	_ttail_search_closest_files_err:
	free(file_sz);
	t->session->file.file_sz = NULL;
	_ttail_search_closest_files_alloc_err:
	free(t->session);
	_ttail_search_closest_files_alloc_session_err:
	return -1;
}

int _ttail_search_closest_files_set_fsizes(ttail_t* t)
{
	size_t i;
	off_t *vfile, *vsz;
	vfile = malloc(sizeof(off_t)*t->logfile_sz);
	if(!vfile)
	{
		perror("Unable to allocate memory for file size sum");
		return -1;
	}
	t->session->file.vfile = vfile;
	vsz = &(t->session->file.vsz);
	*vsz = 0;
	for(i=0; i<t->logfile_sz;i++)
	{
		vfile[i] = *vsz;
		#ifdef HUGEFILE
		*vsz += t->session->file.file_sz[i] >> t->session->file.sz_div;
		#else
		*vsz += t->session->file.file_sz[i];
		#endif
	}
	t->session->file.vpos = 0;
	return 0;
}

int _ttail_file_reopen(ttail_t* t, size_t id)
{
	if(t->logfile[id])
	{
		fclose(t->logfile[id]);
	}
	t->logfile[id] = fopen(t->logfile_name[id], "r");
	if(!t->logfile[id] && t->verbose > 2)
	{
		fprintf(stderr, "Unable to reopen '%s'\n",
			t->logfile_name[id]);
	}
	return t->logfile[id]?0:-1;
}

int _ttail_search_closest_stdin(ttail_t* t)
{
	fprintf(stderr, "Search on stdin not implemented yet");
	return -1;
}

int ttail_logline2date(ttail_t* ttail, const char* logline, struct tm* tm)
{
	ttail_logline_subst(ttail, logline);
	return 0;
}

const char* ttail_logline_subst(ttail_t* ttail, const char* logline)
{
	return logline;
}
