#include <check.h>
#include <libgen.h>

#include "ttail.h"
#include "ttail_init.h"
#include "ttail_search.h"

ttail_t *ttail;

char *samples[5] = { "./samples/1.log", "./samples/2.log", "./samples/3.log",\
	"./samples/4.log", "./samples/5.log"};
off_t samples_sz[5] = { 442, 0, 893, 2587, 2310 };

void teardown_closest(void)
{

	ttail_free(ttail);
}

void setup_closest_fileinit(void)
{
	ttail = ttail_init(1, (char**)&"foo");
	size_t i;
	for(i=0; i<5; i++)
	{
		ttail_add_logfile(ttail, samples[i]);
	}
}

START_TEST (test_search_closest_init)
{
	int ret;

	ret = ttail_search_closest(ttail);
	ck_assert_int_eq(ret,0);
	#ifdef HUGEFILE
	ck_assert_int_eq(ttail->session->file.sz_div,0);
	#endif
}
END_TEST

START_TEST (test_search_closest_init_filesz)
{
	size_t i;
	ttail_search_closest(ttail);
	for(i=0;i<5;i++)
	{
		ck_assert(samples_sz[i] == ttail->session->file.file_sz[i]);
	}

}
END_TEST

START_TEST (test_search_closest_init_vfile)
{
	size_t i;
	off_t full_sz;

	ttail_search_closest(ttail);

	full_sz = 0;
	for(i=0;i<5;i++)
	{
		ck_assert(full_sz == ttail->session->file.vfile[i]);
		full_sz += samples_sz[i];
	}
	ck_assert(full_sz == ttail->session->file.vsz);
}
END_TEST

Suite * ttail_search_suite(void)
{
	Suite *s;
	TCase *tc_search_closest_fileinit;

	s = suite_create("ttail search checks");
	tc_search_closest_fileinit = tcase_create("\
ttail_logline_closest_files_init() checks");
	tcase_add_checked_fixture(tc_search_closest_fileinit,
		setup_closest_fileinit, teardown_closest);
	tcase_add_test(tc_search_closest_fileinit, test_search_closest_init);
	tcase_add_test(tc_search_closest_fileinit,
		test_search_closest_init_filesz);
	tcase_add_test(tc_search_closest_fileinit,
		test_search_closest_init_vfile);

	suite_add_tcase(s, tc_search_closest_fileinit);
	return s;
}

int main(int argc, char **argv)
{
	int number_failed = 0;
	SRunner *sr;

	chdir(dirname(argv[0])); /* move in ./tests dir */

	sr = srunner_create(ttail_search_suite());
	srunner_set_fork_status(sr, CK_FORK);

	srunner_run_all(sr,CK_VERBOSE);
	number_failed = srunner_ntests_failed(sr);
	srunner_free(sr);
	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;

}
